"""
Module realize base ws class
"""

from contextlib import contextmanager
from numbers import Real
from typing import Any, Dict, Optional, Type

from aiohttp import ClientSession
from websocket import create_connection

from .base_api import BaseAPI
from .luna_request import prepareQueryParams


class WSBase:
    """WS Mixin to work with websocket subscription."""

    # service base uri
    baseUri: str
    # get async mode; real method is defined above (BaseAPI)
    getAsyncMode: Type[BaseAPI.getAsyncMode]
    # session object; real property is defined above (BaseHTTPAPI)
    session: ClientSession

    @property
    def _baseWSUri(self) -> str:
        """
        Property get base part of uri for ws.
        """
        return self.baseUri.replace("http", "ws")

    def wsConnect(
        self,
        headers: Dict[str, str],
        queryParams: Dict[str, Any],
        path: str = "/ws",
        messageTimeout: Optional[Real] = None,
        asyncRequest: Optional[bool] = None,
    ):
        """
        Create ws-connection to luna service

        Args:
            headers: request headers
            queryParams: dictionary with queires
            path: ws-connection relative path
            messageTimeout: message receive timeout (time between start waiting message and message appearance)
            asyncRequest: execution in asynchronous mode, disabled by default

        Returns:
            context manager with connection to luna service
        """
        queryParamsDict = prepareQueryParams(queryParams)
        queryParamsStr = "&".join(f"{key}={value}" for key, value in queryParamsDict.items())
        urlWithParams = "{}{}?{}".format(self._baseWSUri, path, queryParamsStr)

        if self.getAsyncMode(asyncRequest):
            return self.session.ws_connect(urlWithParams, headers=headers, receive_timeout=messageTimeout)

        @contextmanager
        def syncWebsocketContextManager():
            webSocketConnection = create_connection(url=urlWithParams, header=headers, timeout=messageTimeout)
            try:
                yield webSocketConnection
            finally:
                webSocketConnection.close()

        return syncWebsocketContextManager()
