"""
Module realize images mixin
"""

from typing import Awaitable, Optional, Union

from luna3.common import functions
from luna3.common.http_objs import BinaryImage
from luna3.common.luna_response import LunaResponse
from luna3.lunavl.mixins.base import MixinStub


class ImagesMixin(MixinStub):
    """Samples Mixin to work with images."""

    def saveImage(
        self, image: BinaryImage, ttl: Optional[int] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Save image to the system.

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/createImages

        Args:
            image: Single image object.
            ttl: Image lifetime
        Returns:
            LunaResponse: body contains json with image_id and url.

        Raises:
            ValueError: if body has unsupported mimetype

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import FACE_IMAGE as image
        >>>
        >>> response = client.saveImage(image=image)
        >>> print(response.statusCode)
        201
        >>> print(json.dumps(response.json, sort_keys=True)) # doctest: +ELLIPSIS
        {"external_url": "http://127.0.0.1:5000/6/images/...", "image_id": ..., "url": ...}

        """
        allowedTypes = ("image/jpeg", "image/png", "image/bmp", "image/tiff", "image/gif", "image/x-portable-pixmap")
        functions.checkContentType(image, allowedTypes=allowedTypes)
        query = {"ttl": ttl}
        payload = functions.createPayloadWithImages(image, {}, self.multipartBoundary)
        return self.makeRequest(f"{self.baseUri}/images", "POST", queryParams=query, body=payload, **kwargs)

    def getImage(
        self, imageId: str, accountIdFilter: Optional[str] = None, withMeta: bool = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get image by id

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/getImage

        Args:
            imageId: image id
            accountIdFilter: account id to get image
            withMeta: whether to retrieve user-defined image metadata

        Returns:
            LunaResponse: body contains image as bytes.

        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import FACE_WARP as image
        >>>
        >>> imageId = client.saveImage(image=image, raiseError=True).json["image_id"]
        >>> response = client.getImage(imageId=imageId)
        >>> print(response.statusCode)
        200
        >>> print(response.contentType)
        image/jpeg
        >>> assert image.body == response.body

        """
        queries = {"account_id": accountIdFilter, "with_meta": withMeta}
        return self.makeRequest(
            f"{self.baseUri}/images/{imageId}", "GET", queryParams=queries, acceptEncoding="identity", **kwargs
        )

    def checkImage(
        self, imageId: str, accountIdFilter: Optional[str] = None, withMeta: bool = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Check to exist an image with id=image_id

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/checkImage

        Args:
            imageId: image id
            accountIdFilter: account id to check image
            withMeta: whether to retrieve user-defined image metadata

        Returns:
            LunaResponse: not contains response body.

        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import FACE_WARP as image
        >>>
        >>> imageId = client.saveImage(image=image, raiseError=True).json["image_id"]
        >>> response = client.checkImage(imageId=imageId)
        >>> print(response.statusCode)
        200

        """
        queries = {"account_id": accountIdFilter, "with_meta": withMeta}
        return self.makeRequest(f"{self.baseUri}/images/{imageId}", "HEAD", queryParams=queries, **kwargs)

    def deleteImage(self, imageId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete image with id=image_id

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/deleteImage

        Args:
            imageId: image id

        Returns:
            LunaResponse: not contains response body.

        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import FACE_WARP as image
        >>>
        >>> imageId = client.saveImage(image=image, raiseError=True).json["image_id"]
        >>> response = client.deleteImage(imageId=imageId)
        >>> print(response.statusCode)
        204

        """
        return self.makeRequest(f"{self.baseUri}/images/{imageId}", "DELETE", **kwargs)
