"""
Module realize Luna-Remote-SDK mixin
"""

from typing import Awaitable, List, Optional, Union

from luna3.common import functions
from luna3.common.http_objs import BinaryImage, UrlForDetection
from luna3.common.luna_response import LunaResponse
from luna3.common.requests import RequestPayload
from luna3.lunavl.mixins.base import MixinStub
from luna3.public.sdk import VideoSDKInput


class LRSMixin(MixinStub):
    """Luna-Remote-SDK Mixin to estimate features on images without side effects."""

    def sdk(
        self,
        inputData: Union[BinaryImage, List[BinaryImage], List[UrlForDetection], List[str]],
        estimatePeopleCount: int = None,
        peopleCountCoordinates: int = None,
        multifacePolicy: int = 1,
        detectFace: Optional[int] = None,
        detectBody: Optional[int] = None,
        estimateHeadPose: Optional[int] = None,
        estimateLandmarks68: Optional[int] = None,
        estimateLandmarks5: Optional[int] = None,
        estimateQuality: Optional[int] = None,
        estimateGaze: Optional[int] = None,
        estimateEyesAttributes: Optional[int] = None,
        estimateEmotions: Optional[int] = None,
        estimateMask: Optional[int] = None,
        estimateLiveness: Optional[int] = None,
        estimateDeepfake: Optional[int] = None,
        estimateGlasses: Optional[int] = None,
        estimateMouthAttributes: Optional[int] = None,
        estimateFaceWarp: Optional[int] = None,
        estimateBodyWarp: Optional[int] = None,
        aggregateAttributes: Optional[int] = None,
        estimateBasicAttributes: Optional[int] = None,
        estimateFaceDescriptor: Optional[int] = None,
        estimateBodyDescriptor: Optional[int] = None,
        estimateUpperBody: Optional[int] = None,
        estimateLowerBody: Optional[int] = None,
        estimateBodyBasicAttributes: Optional[int] = None,
        estimateAccessories: Optional[int] = None,
        pitchThreshold: Optional[int] = None,
        rollThreshold: Optional[int] = None,
        yawThreshold: Optional[int] = None,
        scoreThreshold: float = None,
        maskStates: Optional[List[int]] = None,
        livenessStates: Optional[List[int]] = None,
        deepfakeStates: Optional[List[int]] = None,
        imageType: Optional[int] = None,
        useExifInfo: Optional[int] = None,
        extractExif: Optional[int] = None,
        accept: str = "application/msgpack",
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Detect faces on input images.

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/sdk

        Args:
            inputData: one of:
                        - Single image or images list for extraction (supported formats "jpeg, png, tif, ppm, bmp").
                        - list of urls with images
                        - list of images' ids
            estimatePeopleCount: Available values : 0, 1
            peopleCountCoordinates: Available values : 0, 1
            multifacePolicy: multiple face detection policy:
                             0 - multiple face detection not allowed,
                             1 - multiple face detection allowed,
                             2 - get best detection from the image
            detectFace: detect faces
            detectBody: detect human bodies
            estimateHeadPose: Available values : 0, 1
            estimateLandmarks5: Available values : 0, 1
            estimateLandmarks68: Available values : 0, 1
            estimateQuality: Available values : 0, 1
            estimateGaze: Available values : 0, 1
            estimateEyesAttributes: Available values : 0, 1
            estimateMouthAttributes: Available values : 0, 1
            estimateEmotions: Available values : 0, 1
            estimateMask: Available values : 0, 1
            estimateLiveness: Available values : 0, 1
            estimateDeepfake: Available values : 0, 1
            estimateGlasses: Available values : 0, 1
            estimateFaceDescriptor: Available values : 0, 1
            estimateBasicAttributes: Available values : 0, 1
            estimateBodyDescriptor: Available values : 0, 1
            estimateUpperBody: Available values : 0, 1
            estimateLowerBody: Available values : 0, 1
            estimateBodyBasicAttributes: Available values : 0, 1
            estimateAccessories: Available values : 0, 1
            estimateBodyWarp: Available values : 0, 1
            estimateFaceWarp: Available values : 0, 1
            pitchThreshold: maximum deviation pitch angle from 0
            rollThreshold: maximum deviation roll angle from 0
            yawThreshold: maximum deviation yaw angle from 0
            scoreThreshold: descriptor garbage score
            maskStates: Array of integer (filter by mask state) (Items Enum:1 2 3)
            livenessStates: Array of integer (filter by liveness state) (Items Enum:0 1 2)
            deepfakeStates: Array of integer (filter by deepfake state) (Items Enum:0 1)
            aggregateAttributes: Available values : 0, 1
            imageType: image type (0 - raw image, 1 - face warped image, 2 - body warped image)
            useExifInfo: whether to use exif info for auto orientation
            extractExif: extract EXIF meta information from images
            accept: expected content type
        Returns:
            LunaResponse: body contains json with samples, exif, filename for succeeded images and
            errors for failed images.

        Raises:
            ValueError: if no images were found

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import FACE_WARP as image
        >>>
        >>> response = client.sdk(
        ...     inputData=image,
        ...     estimatePeopleCount=1,
        ...     peopleCountCoordinates=1,
        ...     detectFace=1,
        ...     estimateHeadPose=1,
        ...     estimateEmotions=1,
        ...     estimateGaze=1,
        ...     estimateLiveness=1,
        ...     estimateDeepfake=1,
        ...     estimateQuality=1,
        ...     estimateMask=1,
        ...     estimateEyesAttributes=1,
        ...     estimateMouthAttributes=1,
        ...     estimateGlasses=1,
        ...     estimateFaceWarp=1,
        ...     estimateFaceDescriptor=1,
        ...     estimateLandmarks5=1,
        ...     estimateLandmarks68=1,
        ...     estimateBasicAttributes=1,
        ...     estimateBodyBasicAttributes=1,
        ...     estimateUpperBody=1,
        ...     estimateLowerBody=1,
        ...     estimateAccessories=1,
        ...     pitchThreshold=20,
        ...     rollThreshold=20,
        ...     yawThreshold=20,
        ...     scoreThreshold=0.7,
        ...     livenessStates=[1],
        ...     deepfakeStates=[1],
        ...     aggregateAttributes=1,
        ...     imageType=1,
        ...     useExifInfo=1,
        ...     extractExif=1,
        ...     accept="application/json"
        ... )
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "aggregate_estimations": {
                "body": null,
                "face": {
                    "attributes": {
                        "basic_attributes": {
                            "age": ...,
                            "ethnicities": {
                                "estimations": {
                                    "african_american": ...,
                                    "asian": ...,
                                    "caucasian": ...,
                                    "indian": ...
                                },
                                "predominant_ethnicity": "caucasian"
                            },
                            "gender": 0
                        },
                        "descriptor": {
                            "score": ...,
                            "sdk_descriptor": ...
                        },
                        "emotions": {
                            "estimations": {
                                "anger": ...,
                                "disgust": ...,
                                "fear": ...,
                                "happiness": ...,
                                "neutral": ...,
                                "sadness": ...,
                                "surprise": ...
                            },
                            "predominant_emotion": "neutral"
                        },
                        "mask": {
                            "estimations": {
                                "medical_mask": ...,
                                "missing": ...,
                                "occluded": ...
                            },
                            "face_occlusion": {
                                "estimations": {
                                    "chin": ...,
                                    "clear": ...,
                                    "correct": ...,
                                    "full": ...,
                                    "mouth": ...,
                                    "partially": ...
                                },
                                "predominant_occlusion": "clear"
                            },
                            "predominant_mask": "missing"
                        }
                    }
                }
            },
            "images_estimations": [
                {
                    "estimations": [
                        {
                            "body": null,
                            "face": {
                                "detection": {
                                    "attributes": {
                                        "emotions": {
                                            "estimations": {
                                                "anger": ...,
                                                "disgust": ...,
                                                "fear": ...,
                                                "happiness": ...,
                                                "neutral": ...,
                                                "sadness": ...,
                                                "surprise": ...
                                            },
                                            "predominant_emotion": "neutral"
                                        },
                                        "glasses": {
                                            "glasses": "no_glasses"
                                        },
                                        "mask": {
                                            "estimations": {
                                                "medical_mask": ...,
                                                "missing": ...,
                                                "occluded": ...
                                            },
                                            "face_occlusion": {
                                                "estimations": {
                                                    "chin": ...,
                                                    "clear": ...,
                                                    "correct": ...,
                                                    "full": ...,
                                                    "mouth": ...,
                                                    "partially": ...
                                                },
                                                "predominant_occlusion": "clear"
                                            },
                                            "predominant_mask": "missing"
                                        },
                                        "mouth_attributes": {
                                            "occluded": ...,
                                            "opened": ...,
                                            "score": ...,
                                            "smile": ...
                                        }
                                    },
                                    "quality": {
                                        "blurriness": ...,
                                        "dark": ...,
                                        "illumination": ...,
                                        "light": ...,
                                        "specularity": ...
                                    },
                                    "warp": ...
                                }
                            }
                        }
                    ],
                    "exif": {},
                    "filename": "raw image",
                    "image_estimations": {
                        "people": {
                            "coordinates": ...,
                            "count": ...
                        }
                    }
                }
            ]
        }

        """
        queries = {
            "multiface_policy": multifacePolicy,
            "estimate_people_count": estimatePeopleCount,
            "people_count_coordinates": peopleCountCoordinates,
            "estimate_head_pose": estimateHeadPose,
            "estimate_landmarks68": estimateLandmarks68,
            "estimate_landmarks5": estimateLandmarks5,
            "estimate_quality": estimateQuality,
            "estimate_gaze": estimateGaze,
            "estimate_eyes_attributes": estimateEyesAttributes,
            "estimate_mouth_attributes": estimateMouthAttributes,
            "estimate_emotions": estimateEmotions,
            "estimate_mask": estimateMask,
            "estimate_liveness": estimateLiveness,
            "estimate_deepfake": estimateDeepfake,
            "estimate_glasses": estimateGlasses,
            "pitch_threshold": pitchThreshold,
            "roll_threshold": rollThreshold,
            "yaw_threshold": yawThreshold,
            "image_type": imageType,
            "estimate_body_descriptor": estimateBodyDescriptor,
            "estimate_upper_body": estimateUpperBody,
            "estimate_lower_body": estimateLowerBody,
            "estimate_body_basic_attributes": estimateBodyBasicAttributes,
            "estimate_accessories": estimateAccessories,
            "estimate_body_warp": estimateBodyWarp,
            "estimate_face_warp": estimateFaceWarp,
            "aggregate_attributes": aggregateAttributes,
            "estimate_face_descriptor": estimateFaceDescriptor,
            "score_threshold": scoreThreshold,
            "estimate_basic_attributes": estimateBasicAttributes,
            "detect_face": detectFace,
            "detect_body": detectBody,
            "mask_states": maskStates,
            "liveness_states": livenessStates,
            "deepfake_states": deepfakeStates,
            "use_exif_info": useExifInfo,
            "extract_exif": extractExif,
        }
        payload = functions.createPayloadWithImages(inputData, {}, self.multipartBoundary)
        if accept:
            payload.applyAcceptHeader(accept)
        return self.makeRequest(f"{self.baseUri}/sdk", "POST", queryParams=queries, body=payload, **kwargs)

    def checkISO(
        self,
        inputData: Union[BinaryImage, List[BinaryImage], List[UrlForDetection], List[str]],
        multifacePolicy: int = 0,
        extractExif: int = 0,
        imageType: Optional[int] = None,
        accept: str = "application/msgpack",
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Estimate and check input images by ISO.

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#tag/iso

        Args:
            inputData: one of:
                        - Single image or images list for extraction (supported formats "jpeg, png, tif, ppm, bmp").
                        - list of urls with images
                        - list of images' ids
            multifacePolicy: multiple face detection policy:
                             0 - multiple face detection not allowed,
                             1 - multiple face detection allowed
                             2 - get best detection from the image
            extractExif: Available values : 0, 1
            imageType: image type (0 - raw image, 1 - face warped image)
            accept: expected content type

        Returns:
            LunaResponse: body contains json with filename, exif, error and estimations for images

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import BODY_IMAGE as image
        >>>
        >>> response = client.checkISO(
        ...     inputData=image, multifacePolicy=1, extractExif=1, accept="application/json"
        ... )
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "images": [
                {
                    "error": {
                        "desc": "Success",
                        "detail": "Success",
                        "error_code": 0,
                        "link": "https://docs.visionlabs.ai/info/luna/troubleshooting/errors-description/code-0"
                    },
                    "estimations": [
                        {
                            "face": {
                                "detection": {
                                    "iso": {
                                        "checks": [
                                            {
                                                "name": "image_format",
                                                "object_value": "JPEG",
                                                "result": 1,
                                                "threshold_value": [
                                                    "JPEG",
                                                    "JPEG2000",
                                                    "PNG"
                                                ]
                                            },
                                            {
                                                "name": "illumination_quality",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": {
                                                    "max": 1,
                                                    "min": 0.3
                                                }
                                            },
                                            {
                                                "name": "specularity_quality",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": {
                                                    "max": 1,
                                                    "min": 0.3
                                                }
                                            },
                                            {
                                                "name": "blurriness_quality",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": {
                                                    "max": 1,
                                                    "min": 0.61
                                                }
                                            },
                                            {
                                                "name": "dark_quality",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": {
                                                    "max": 1,
                                                    "min": 0.5
                                                }
                                            },
                                            {
                                                "name": "light_quality",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": {
                                                    "max": 1,
                                                    "min": 0.57
                                                }
                                            },
                                            {
                                                "name": "head_yaw",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": {
                                                    "max": 5.0,
                                                    "min": -5.0
                                                }
                                            },
                                            {
                                                "name": "head_pitch",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": {
                                                    "max": 5.0,
                                                    "min": -5.0
                                                }
                                            },
                                            {
                                                "name": "head_roll",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": {
                                                    "max": 8.0,
                                                    "min": -8.0
                                                }
                                            },
                                            {
                                                "name": "gaze_yaw",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": {
                                                    "max": 5.0,
                                                    "min": -5.0
                                                }
                                            },
                                            {
                                                "name": "gaze_pitch",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": {
                                                    "max": 5.0,
                                                    "min": -5.0
                                                }
                                            },
                                            {
                                                "name": "mouth_smiling",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": {
                                                    "max": 0.5,
                                                    "min": 0
                                                }
                                            },
                                            {
                                                "name": "mouth_occluded",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": {
                                                    "max": 0.5,
                                                    "min": 0
                                                }
                                            },
                                            {
                                                "name": "mouth_open",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": {
                                                    "max": 0.5,
                                                    "min": 0
                                                }
                                            },
                                            {
                                                "name": "glasses",
                                                "object_value": "no_glasses",
                                                "result": 1,
                                                "threshold_value": [
                                                    "no_glasses",
                                                    "eyeglasses"
                                                ]
                                            },
                                            {
                                                "name": "left_eye",
                                                "object_value": "open",
                                                "result": 1,
                                                "threshold_value": [
                                                    "open"
                                                ]
                                            },
                                            {
                                                "name": "right_eye",
                                                "object_value": "open",
                                                "result": 1,
                                                "threshold_value": [
                                                    "open"
                                                ]
                                            },
                                            {
                                                "name": "head_horizontal_center",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": {
                                                    "max": 0.55,
                                                    "min": 0.45
                                                }
                                            },
                                            {
                                                "name": "head_vertical_center",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": {
                                                    "max": 0.5,
                                                    "min": 0.3
                                                }
                                            },
                                            {
                                                "name": "head_width",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": {
                                                    "max": 0.75,
                                                    "min": 0.5
                                                }
                                            },
                                            {
                                                "name": "head_height",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": {
                                                    "max": 0.9,
                                                    "min": 0.6
                                                }
                                            },
                                            {
                                                "name": "eye_distance",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": {
                                                    "max": null,
                                                    "min": 90.0
                                                }
                                            },
                                            {
                                                "name": "eyebrows_state",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": [
                                                    "neutral"
                                                ]
                                            },
                                            {
                                                "name": "shoulders_position",
                                                "object_value": "parallel",
                                                "result": 1,
                                                "threshold_value": [
                                                    "parallel"
                                                ]
                                            },
                                            {
                                                "name": "smile_properties",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": [
                                                    "none"
                                                ]
                                            },
                                            {
                                                "name": "headwear_type",
                                                "object_value": "none",
                                                "result": 1,
                                                "threshold_value": [
                                                    "none"
                                                ]
                                            },
                                            {
                                                "name": "natural_light",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": 1
                                            },
                                            {
                                                "name": "radial_distortion",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": 0
                                            },
                                            {
                                                "name": "red_eyes",
                                                "object_value": 0,
                                                "result": 1,
                                                "threshold_value": 0
                                            },
                                            {
                                                "name": "face_color_type",
                                                "object_value": "color",
                                                "result": 1,
                                                "threshold_value": [
                                                    "grayscale",
                                                    "color"
                                                ]
                                            },
                                            {
                                                "name": "background_uniformity",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": {
                                                    "max": 1,
                                                    "min": 0.5
                                                }
                                            },
                                            {
                                                "name": "background_lightness",
                                                "object_value": ...,
                                                "result": ...,
                                                "threshold_value": {
                                                    "max": 1,
                                                    "min": 0.2
                                                }
                                            }
                                        ],
                                        "status": 0
                                    },
                                    "rect": {
                                        "height": ...,
                                        "width": ...,
                                        "x": ...,
                                        "y": ...
                                    }
                                }
                            }
                        }
                    ],
                    "exif": {},
                    "filename": "raw image",
                    "status": 1
                }
            ]
        }

        """
        queries = {"multiface_policy": multifacePolicy, "image_type": imageType, "extract_exif": extractExif}

        payload = functions.createPayloadWithImages(inputData, {}, self.multipartBoundary)
        if accept:
            payload.applyAcceptHeader(accept)

        return self.makeRequest(f"{self.baseUri}/iso", "POST", queryParams=queries, body=payload, **kwargs)

    def videosdk(
        self, inputData: VideoSDKInput, accept: str = "application/msgpack", **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Process video

        Args:
            inputData: video properties including url and analytics params

        Returns:
            LunaResponse: body contains json with analytics results
            errors for failed images.


        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from luna3.public.sdk import VideoSDKInput, VideoSDKAnalytics, VideoSDKProperties
        >>> from luna3.public.sdk import PeopleCountAnalyticsParams, Rate
        >>> url ="https://lunatestttttt.s3.eu-central-1.amazonaws.com/video_test.avi"
        >>> params = PeopleCountAnalyticsParams(rate=Rate(unit="second", period=1), probeCount=2)
        >>> analytics = [VideoSDKAnalytics(name="people_count", targets=[], parameters=params)]
        >>> response = client.videosdk(inputData=VideoSDKInput(VideoSDKProperties(url), analytics))
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "analytics": [
                {
                    "name": "people_count",
                    "result": {
                        "crowd": {
                            "events": [
                                {
                                    "event_id": ...,
                                    "frames_estimations": [
                                        {
                                            "frame_offset": ...,
                                            "people_count": ...,
                                            "time_offset": ...
                                        },
                                        {
                                            "frame_offset": ...,
                                            "people_count": ...,
                                            "time_offset": ...
                                        },
                                        {
                                            "frame_offset": ...,
                                            "people_count": ...,
                                            "time_offset": ...
                                        },
                                        {
                                            "frame_offset": ...,
                                            "people_count": ...,
                                            "time_offset": ...
                                        },
                                        {
                                            "frame_offset": ...,
                                            "people_count": ...,
                                            "time_offset": ...
                                        }
                                    ],
                                    "max_people_count": 2,
                                    "video_segment": {
                                        "end_time_offset": ...,
                                        "last_frame_offset": ...,
                                        "start_frame_offset": ...,
                                        "start_time_offset": ...
                                    }
                                }
                            ]
                        }
                    }
                }
            ],
            "video_data": {
                "duration": 5.0,
                "frame_rate": 24.0,
                "frames_count": 120
            }
        }
        """
        payload = RequestPayload.buildJson(inputData.asDict())
        return self.makeRequest(f"{self.baseUri}/videosdk", "POST", body=payload, accept=accept, **kwargs)
