"""
Module realize samples mixin
"""

from typing import Awaitable, List, Optional, Union

from luna3.common import functions
from luna3.common.http_objs import BinaryImage, UrlForDetection
from luna3.common.luna_response import LunaResponse
from luna3.lunavl.mixins.base import MixinStub


class SamplesMixin(MixinStub):
    """Samples Mixin to work with samples."""

    def detectFaces(
        self,
        inputData: Union[BinaryImage, List[BinaryImage], List[UrlForDetection], List[str]],
        multifacePolicy: int = 1,
        estimateHeadPose: int = 0,
        detectLandmarks68: int = 0,
        estimateQuality: int = 0,
        estimateGaze: int = 0,
        estimateEyesAttributes: int = 0,
        estimateEmotions: int = 0,
        estimateMask: int = 0,
        estimateMouthAttributes: int = 0,
        pitchThreshold: Optional[int] = None,
        rollThreshold: Optional[int] = None,
        yawThreshold: Optional[int] = None,
        warpedImage: int = 0,
        extractExif: Optional[int] = None,
        useExifInfo: Optional[int] = None,
        estimateFaceQuality: int = 0,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Detect faces on input images.

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/detectFaces

        Args:
            inputData: one of:
                        - Single image or images list for extraction (supported formats "jpeg, png, tif, ppm, bmp").
                        - list of urls with images
                        - list of images' ids
            multifacePolicy: multiple face detection policy:
                             0 - multiple face detection not allowed,
                             1 - multiple face detection allowed,
                             2 - get best detection from the image
            estimateHeadPose: Available values : 0, 1
            detectLandmarks68: Available values : 0, 1
            estimateQuality: Available values : 0, 1
            estimateGaze: Available values : 0, 1
            estimateEyesAttributes: Available values : 0, 1
            estimateMouthAttributes: Available values : 0, 1
            estimateEmotions: Available values : 0, 1
            estimateMask: Available values : 0, 1
            pitchThreshold: maximum deviation pitch angle from 0
            rollThreshold: maximum deviation roll angle from 0
            yawThreshold: maximum deviation yaw angle from 0
            warpedImage: Whether input image is a warped or arbitrary image.
            extractExif: Whether to extract exif from the image
            useExifInfo: whether to use exif info for auto orientation
            estimateFaceQuality: whether to estimate face quality

        Returns:
             LunaResponse: body contains json with samples, exif, filename for succeeded images and
             errors for failed images.

        Raises:
            ValueError: if no images were found

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import FACE_IMAGE
        >>>
        >>> response = client.detectFaces(
        ...     inputData=FACE_IMAGE,
        ...     multifacePolicy=1,
        ...     estimateHeadPose=1,
        ...     estimateEmotions=1,
        ...     estimateQuality=1,
        ...     estimateGaze=1,
        ...     estimateMask=1,
        ...     extractExif=1,
        ...     estimateEyesAttributes=1,
        ...     estimateMouthAttributes=1,
        ...     pitchThreshold=20,
        ...     rollThreshold=20,
        ...     yawThreshold=20,
        ...     useExifInfo=1,
        ...     estimateFaceQuality=1,
        ... )
        >>> print(response.statusCode)
        201
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        {
            "images": [
                {
                    "detections": {
                        "filtered_detections": {
                            "face_detections": [
                                {
                                    "detection": {
                                        "attributes": {
                                            "emotions": {
                                                "estimations": {
                                                    "anger": ...,
                                                    "disgust": ...,
                                                    "fear": ...,
                                                    "happiness": ...,
                                                    "neutral": ...,
                                                    "sadness": ...,
                                                    "surprise": ...
                                                },
                                                "predominant_emotion": "neutral"
                                            },
                                            "eyes_attributes": {
                                                "left_eye": {
                                                    "iris_landmarks": [...],
                                                    "state": "open"
                                                },
                                                "right_eye": {
                                                    "iris_landmarks": [...],
                                                    "state": "open"
                                                }
                                            },
                                            "gaze": {
                                                "pitch": ...,
                                                "yaw": ...
                                            },
                                            "head_pose": {
                                                "pitch": ...,
                                                "roll": ...,
                                                "yaw": ...
                                            },
                                            "mask": {
                                                "estimations": {
                                                    "medical_mask": ...,
                                                    "missing": ...,
                                                    "occluded": ...
                                                },
                                                "face_occlusion": {
                                                    "estimations": {
                                                        "chin": ...,
                                                        "clear": ...,
                                                        "correct": ...,
                                                        "full": ...,
                                                        "mouth": ...,
                                                        "partially": ...
                                                    },
                                                    "predominant_occlusion": "clear"
                                                },
                                                "predominant_mask": "missing"
                                            },
                                            "mouth_attributes": {
                                                "occluded": ...,
                                                "opened": ...,
                                                "score": ...,
                                                "smile": ...
                                            }
                                        },
                                        "face_quality": {
                                            "checks": [...],
                                            "status": 0
                                        },
                                        "landmarks5": [...],
                                        "quality": {
                                            "blurriness": ...,
                                            "dark": ...,
                                            "illumination": ...,
                                            "light": ...,
                                            "specularity": ...
                                        },
                                        "rect": {
                                            "height": ...,
                                            "width": ...,
                                            "x": ...,
                                            "y": ...
                                        }
                                    },
                                    "filter_reasons": [...]
                                }
                            ]
                        },
                        "samples": []
                    },
                    "error": {
                        "desc": "Success",
                        "detail": "Success",
                        "error_code": 0,
                        "link": "https://docs.visionlabs.ai/info/luna/troubleshooting/errors-description/code-0"
                    },
                    "exif": {},
                    "filename": "raw_image",
                    "status": 1
                }
            ]
        }

        """
        queries = {
            "multiface_policy": multifacePolicy,
            "estimate_head_pose": estimateHeadPose,
            "detect_landmarks68": detectLandmarks68,
            "estimate_quality": estimateQuality,
            "estimate_gaze": estimateGaze,
            "estimate_eyes_attributes": estimateEyesAttributes,
            "estimate_mouth_attributes": estimateMouthAttributes,
            "estimate_emotions": estimateEmotions,
            "estimate_mask": estimateMask,
            "pitch_threshold": pitchThreshold,
            "roll_threshold": rollThreshold,
            "yaw_threshold": yawThreshold,
            "warped_image": warpedImage,
            "extract_exif": extractExif,
            "use_exif_info": useExifInfo,
            "estimate_face_quality": estimateFaceQuality,
        }

        payload = functions.createPayloadWithImages(inputData, {}, self.multipartBoundary)
        return self.makeRequest(f"{self.baseUri}/detector", "POST", queryParams=queries, body=payload, **kwargs)

    def saveFaceSample(
        self, image: BinaryImage, ttl: Optional[int] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Save face sample to the system.

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/saveSample

        Args:
            image: Single image object.
            ttl: Sample lifetime

        Returns:
           LunaResponse: body contains json with sample_id and url.

        Raises:
            ValueError: if body has unsupported mimetype.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import FACE_WARP as image
        >>>
        >>> response = client.saveFaceSample(image=image)
        >>> print(response.statusCode)
        201
        >>> print(json.dumps(response.json, sort_keys=True))  # doctest: +ELLIPSIS
        {"sample_id": ..., "url": ...}

        """
        allowedTypes = ("image/jpeg", "image/png", "image/bmp", "image/tiff", "image/gif", "image/x-portable-pixmap")
        functions.checkContentType(image, allowedTypes=allowedTypes)

        query = {"ttl": ttl}
        payload = functions.createPayloadWithImages(image, {}, self.multipartBoundary)
        return self.makeRequest(f"{self.baseUri}/samples/faces", "POST", queryParams=query, body=payload, **kwargs)

    def getFaceSample(
        self, sampleId: str, accountIdFilter: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get face sample by id

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/getSample

        Args:
            sampleId: sample Id
            accountIdFilter: account id to get sample

        Returns:
            LunaResponse: body contains image as bytes.

        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import FACE_WARP as image
        >>>
        >>> sampleId = client.saveFaceSample(image=image, raiseError=True).json["sample_id"]
        >>> response = client.getFaceSample(sampleId=sampleId)
        >>> print(response.statusCode)
        200
        >>> print(response.contentType)
        image/jpeg
        >>> assert image.body == response.body

        """
        queries = {"account_id": accountIdFilter}
        return self.makeRequest(
            f"{self.baseUri}/samples/faces/{sampleId}", "GET", queryParams=queries, acceptEncoding="identity", **kwargs
        )

    def checkFaceSample(
        self, sampleId: str, accountIdFilter: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Check to exist a face sample with id=sample_id

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/checkSample

        Args:
            sampleId: sample Id
            accountIdFilter: account id to check sample

        Returns:
            LunaResponse: not contains response body.

        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import FACE_WARP as image
        >>>
        >>> sampleId = client.saveFaceSample(image=image, raiseError=True).json["sample_id"]
        >>> response = client.checkFaceSample(sampleId=sampleId)
        >>> print(response.statusCode)
        200

        """
        queries = {"account_id": accountIdFilter}
        return self.makeRequest(f"{self.baseUri}/samples/faces/{sampleId}", "HEAD", queryParams=queries, **kwargs)

    def deleteFaceSample(self, sampleId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete face sample with id=sample_id

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/deleteSample

        Args:
            sampleId: sample Id

        Returns:
            LunaResponse: not contains response body.

        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import FACE_WARP as image
        >>>
        >>> sampleId = client.saveFaceSample(image=image, raiseError=True).json["sample_id"]
        >>> response = client.deleteFaceSample(sampleId=sampleId)
        >>> print(response.statusCode)
        204

        """
        return self.makeRequest(f"{self.baseUri}/samples/faces/{sampleId}", "DELETE", **kwargs)

    def saveBodySample(
        self, image: BinaryImage, ttl: Optional[int] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Save body sample to the system.

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/saveSample

        Args:
            image: Single image object.
            ttl: Object lifetime

        Returns:
            LunaResponse: body contains json with sample_id and url.

        Raises:
            ValueError: if body has unsupported mimetype.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import BODY_WARP as image
        >>>
        >>> response = client.saveBodySample(image=image)
        >>> print(response.statusCode)
        201
        >>> print(json.dumps(response.json, sort_keys=True))  # doctest: +ELLIPSIS
        {"sample_id": ..., "url": ...}

        """
        allowedTypes = ("image/jpeg", "image/png", "image/bmp", "image/tiff", "image/gif", "image/x-portable-pixmap")
        functions.checkContentType(image, allowedTypes=allowedTypes)

        query = {"ttl": ttl}
        payload = functions.createPayloadWithImages(image, {}, self.multipartBoundary)
        return self.makeRequest(f"{self.baseUri}/samples/bodies", "POST", queryParams=query, body=payload, **kwargs)

    def getBodySample(
        self, sampleId: str, accountIdFilter: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get body sample by id

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/getSample

        Args:
            sampleId: sample Id
            accountIdFilter: account id to get sample

        Returns:
            LunaResponse: body contains image as bytes.

        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import BODY_WARP as image
        >>>
        >>> sampleId = client.saveBodySample(image=image, raiseError=True).json["sample_id"]
        >>> response = client.getBodySample(sampleId=sampleId)
        >>> print(response.statusCode)
        200
        >>> print(response.contentType)
        image/jpeg
        >>> assert image.body == response.body

        """
        queries = {"account_id": accountIdFilter}
        return self.makeRequest(
            f"{self.baseUri}/samples/bodies/{sampleId}", "GET", queryParams=queries, acceptEncoding="identity", **kwargs
        )

    def checkBodySample(
        self, sampleId: str, accountIdFilter: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Check to exist a body sample with id=sample_id

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/checkSample

        Args:
            sampleId: sample Id
            accountIdFilter: account id to check sample

        Returns:
            LunaResponse: not contains response body.

        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import BODY_WARP as image
        >>>
        >>> sampleId = client.saveBodySample(image=image, raiseError=True).json["sample_id"]
        >>> response = client.checkBodySample(sampleId=sampleId)
        >>> print(response.statusCode)
        200

        """
        queries = {"account_id": accountIdFilter}
        return self.makeRequest(f"{self.baseUri}/samples/bodies/{sampleId}", "HEAD", queryParams=queries, **kwargs)

    def deleteBodySample(self, sampleId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete body sample with id=sample_id

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/deleteSample

        Args:
            sampleId: sample Id

        Returns:
            LunaResponse: not contains response body.

        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import BODY_WARP as image
        >>>
        >>> sampleId = client.saveBodySample(image=image, raiseError=True).json["sample_id"]
        >>> response = client.deleteBodySample(sampleId=sampleId)
        >>> print(response.statusCode)
        204

        """
        return self.makeRequest(f"{self.baseUri}/samples/bodies/{sampleId}", "DELETE", **kwargs)
